/*
    qtparted - a frontend to libparted for manipulating disk partitions
    Copyright (C) 2002-2003 Vanni Brutto

    Vanni Brutto <zanac (-at-) libero dot it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include "qp_dlgresize.h"

QP_dlgResize::QP_dlgResize(QWidget *, const char *) {

    sizecontainer = NULL;
    
    /*---this is a fixed dialog!---*/
    setFixedSize(minimumSizeHint());
    
    sizecontainer = new QP_SizeContainer(container);
    connect(sizecontainer, SIGNAL(sigChangedStart(PedSector)),
            this, SLOT(slotChangedStart(PedSector)));
    connect(sizecontainer, SIGNAL(sigChangedEnd(PedSector)),
            this, SLOT(slotChangedEnd(PedSector)));
    connect(sizecontainer, SIGNAL(sigChangedPos(PedSector, PedSector)),
            this, SLOT(slotChangedPos(PedSector, PedSector)));

    connect(spinFreeBefore, SIGNAL(valueChanged(int)),
            this, SLOT(slotFreeBeforeChanged(int)));
    connect(spinFreeAfter, SIGNAL(valueChanged(int)),
            this, SLOT(slotFreeAfterChanged(int)));
    connect(spinNewSize, SIGNAL(valueChanged(int)),
            this, SLOT(slotNewSizeChanged(int)));
}

QP_dlgResize::~QP_dlgResize() {
}

void QP_dlgResize::init_dialog(QTParted::MoveResize moveresize) {
    _moveresize = moveresize;

    sizecontainer->setMode(_moveresize);
    sizecontainer->setGrowStartPartSector(_GrowStartPartSector);
    sizecontainer->setGrowEndPartSector(_GrowEndPartSector);
    sizecontainer->setStartPartSector(_StartPartSector);
    sizecontainer->setEndPartSector(_EndPartSector);
    sizecontainer->setMinPartSector(_MinPartSector);

    QString label;
    label = QString("Minimum Size: %1 MB")
            .arg((int)_MinPartSector/MEGABYTE_SECTORS);
    lblMinimumSize->setText(label);

    spinFreeBefore->setEnabled(true);
    spinFreeAfter->setEnabled(true);
    spinNewSize->setEnabled(true);

    setValNewSize();
    setValFreeBefore();
    setValFreeAfter();

    if (_moveresize == QTParted::resize) {
        spinFreeBefore->setEnabled(false);
        setCaption("Resize partition");
    } else {
        spinNewSize->setEnabled(false);
        setCaption("Move partition");
    }

    sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
}

int QP_dlgResize::show_dialog() {
    return exec();
}

void QP_dlgResize::resizeEvent(QResizeEvent *) {
}

void QP_dlgResize::setMaxPartSector(PedSector MaxPartSector) {
    _MaxPartSector = MaxPartSector;
}

void QP_dlgResize::setMinPartSector(PedSector MinPartSector) {
    _MinPartSector = MinPartSector;
}

void QP_dlgResize::setStartPartSector(PedSector StartPartSector) {
    _StartPartSector = StartPartSector;
}

void QP_dlgResize::setEndPartSector(PedSector EndPartSector) {
    _EndPartSector = EndPartSector;
}

void QP_dlgResize::setGrowStartPartSector(PedSector GrowStartPartSector) {
    _GrowStartPartSector = GrowStartPartSector;
}

void QP_dlgResize::setGrowEndPartSector(PedSector GrowEndPartSector) {
    _GrowEndPartSector = GrowEndPartSector;
}

void QP_dlgResize::setValFreeBefore() {
    PedSector size = _EndPartSector - _StartPartSector;
    int maxfree = (int)((_GrowEndPartSector - _GrowStartPartSector - size)/MEGABYTE_SECTORS);
    int sizebefore = (int)((_StartPartSector - _GrowStartPartSector)/MEGABYTE_SECTORS);
    if (maxfree < 0) maxfree = 0;

    spinFreeBefore->blockSignals(true);
    spinFreeBefore->setMaxValue(maxfree);
    spinFreeBefore->setValue(sizebefore);
    spinFreeBefore->blockSignals(false);
}

void QP_dlgResize::setValFreeAfter() {
    int maxfree;
    
    if (_moveresize == QTParted::resize) {
        maxfree = (int)((_GrowEndPartSector - (_StartPartSector + _MinPartSector))/MEGABYTE_SECTORS);
    } else {
        PedSector size = _EndPartSector - _StartPartSector;
        maxfree = (int)((_GrowEndPartSector - _GrowStartPartSector - size)/MEGABYTE_SECTORS);
    }
    int sizeafter = (int)((_GrowEndPartSector - _EndPartSector)/MEGABYTE_SECTORS);
    if (maxfree < 0) maxfree = 0;

    spinFreeAfter->blockSignals(true);
    spinFreeAfter->setMaxValue(maxfree);
    spinFreeAfter->setValue(sizeafter);
    spinFreeAfter->blockSignals(false);
}

void QP_dlgResize::setValNewSize() {
    int valnewsize = (int)(_EndPartSector/MEGABYTE_SECTORS - _StartPartSector/MEGABYTE_SECTORS);
    int maxval = (int)(_GrowEndPartSector/MEGABYTE_SECTORS - _StartPartSector/MEGABYTE_SECTORS);

    spinNewSize->blockSignals(true);
    spinNewSize->setMaxValue(maxval);
    spinNewSize->setMinValue((int)_MinPartSector/MEGABYTE_SECTORS);
    spinNewSize->setValue(valnewsize);
    spinNewSize->blockSignals(false);
}

void QP_dlgResize::slotChangedStart(PedSector StartSector) {
    _StartPartSector = StartSector;

    setValNewSize();
    setValFreeBefore();
}

void QP_dlgResize::slotChangedEnd(PedSector EndSector) {
    _EndPartSector = EndSector;
    
    setValNewSize();
    setValFreeAfter();
}

void QP_dlgResize::slotChangedPos(PedSector StartSector, PedSector EndSector) {
    PedSector size = _EndPartSector - _StartPartSector;
    _StartPartSector = StartSector;
    _EndPartSector = StartSector + size;

    setValFreeAfter();
    setValFreeBefore();
}

void QP_dlgResize::slotFreeBeforeChanged(int FreeBefore) {
    if (_moveresize == QTParted::resize) {
        sizecontainer->blockSignals(true);
        _StartPartSector = FreeBefore*MEGABYTE_SECTORS + _GrowStartPartSector;
        sizecontainer->setStartPartSector(_StartPartSector);
        sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
        sizecontainer->blockSignals(false);
    } else {
        PedSector size = _EndPartSector - _StartPartSector;
        _StartPartSector = _GrowStartPartSector + FreeBefore*MEGABYTE_SECTORS;
        _EndPartSector = _StartPartSector + size;
        
        int freeafter = (_GrowEndPartSector - _EndPartSector)/MEGABYTE_SECTORS;

        spinFreeAfter->setValue(freeafter);

        sizecontainer->blockSignals(true);
        sizecontainer->setStartPartSector(_StartPartSector);
        sizecontainer->setEndPartSector(_EndPartSector);
        sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
        sizecontainer->blockSignals(false);
    }
}

void QP_dlgResize::slotFreeAfterChanged(int FreeAfter) {
    if (_moveresize == QTParted::resize) {
        sizecontainer->blockSignals(true);
        _EndPartSector = _GrowEndPartSector - FreeAfter*MEGABYTE_SECTORS;
        sizecontainer->setEndPartSector(_EndPartSector);
        sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
        sizecontainer->blockSignals(false);

        setValNewSize();
    } else {
        PedSector size = _EndPartSector - _StartPartSector;
        _EndPartSector = _GrowEndPartSector - FreeAfter*MEGABYTE_SECTORS;
        _StartPartSector = _EndPartSector - size;
        
        int freebefore = (_StartPartSector - _GrowStartPartSector)/MEGABYTE_SECTORS;

        spinFreeBefore->setValue(freebefore);

        sizecontainer->blockSignals(true);
        sizecontainer->setStartPartSector(_StartPartSector);
        sizecontainer->setEndPartSector(_EndPartSector);
        sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
        sizecontainer->blockSignals(false);
    }
}

void QP_dlgResize::slotNewSizeChanged(int NewSize) {
    sizecontainer->blockSignals(true);
    
    _EndPartSector = _StartPartSector + NewSize*MEGABYTE_SECTORS;
    sizecontainer->setEndPartSector(_EndPartSector);
    sizecontainer->setGeometry(2, 2, container->width()-4, container->height()-4);
    
    sizecontainer->blockSignals(false);

    setValFreeAfter();
}
